from .base import BaseSearch


class Katcr(BaseSearch):
    """KickAssTorrents search engine.

    Search on any of the pirate bay proxies or thepiratebay itself if
    available. Extract torrents, description and return it tabulated and ready
    to be printed
    """

    proxy_name = 'Kickass'
    url_format = '{}{}/usearch/{}/{}/'
    url = 'https://kat.cd'



from .katcr import Katcr  # noqa
from .tpb import ThePirateBay  # noqa
from .nyaasi import NyaaSi  # noqa
from .skytorrents import Skytorrents  # noqa
from .eztv import Eztv  # noqa
from .jackett import Jackett  # noqa



#!/usr/bin/env python3
"""Python library and CLI to search in multiple torrent sources.

Uses torrentmirror to get the first open mirror for any of the
supported sites.
"""

import os
import xdg
import warnings
import shutil
import subprocess
import asyncio
import urllib.parse
from configparser import ConfigParser

import cutie
import aiohttp
from cleo import Command
from cleo import Application
from pygogo import Gogo

from . import engines
from .bot import BotHandler
from .engines.base import BaseSearch

try:
    from torrentstream import stream_torrent
except ImportError:
    warnings.warn("Could not import torrentstream, streaming is disabled")

    def stream_torrent(torrent):
        """Stream torrents"""


CONFIG_FILE = xdg.XDG_CONFIG_HOME / 'katcr.ini'
DEFAULT_ENGINES = ['Katcr', 'ThePirateBay']
MAX_SIZE = shutil.get_terminal_size().columns - 20


class Result:
    """Represents a result, results are shortened and cleanly printable."""
    session = None

    def __init__(self, result, interactive):
        self.result = result
        self.is_interactive = interactive

    def __repr__(self):
        """Get new search res from shortened urls and adjust term output."""
        if self.is_interactive:
            return self.result[0][:MAX_SIZE]
        return ' | '.join(self.result)

    def open(self):
        """Open with xdg-open"""
        return subprocess.check_call(['xdg-open', self.result[-1]])


class SearcherCommand(Command):
    async def setup_sessions(self):
        """Setup client session as singleton"""
        self.logger = Gogo(__name__, verbose=self.io.verbosity).logger
        BaseSearch.session = aiohttp.ClientSession(raise_for_status=True)
        BaseSearch.logger = self.logger
        BaseSearch.config = ConfigParser()
        BaseSearch.config.read(CONFIG_FILE)
        Result.session = BaseSearch.session

    @staticmethod
    async def teardown_sessions():
        await BaseSearch.session.close()

    async def search(self, enames, search_term, pages, is_interactive, stream):
        """Search on all engines."""

        virtual = [a for a in enames if ':' in a]
        real = [a for a in enames if ':' not in a]

        veng = list(
            getattr(engines, a[0])(a[1])
            for a in [a.split(':') for a in virtual])
        eng = list(getattr(engines, a)() for a in real)

        search_term = urllib.parse.quote(search_term)
        await self.setup_sessions()
        search_res = []

        for engine in veng + eng:
            engine_result = await engine.search(search_term, int(pages))
            search_res.extend(
                (Result(a, is_interactive) for a in engine_result))

        await self.teardown_sessions()
        if not search_res:
            return

        if not is_interactive:
            self.render_table(['Description', 'Link'],
                              [a.result for a in search_res])
            return search_res

        result = search_res[cutie.select(search_res, selected_prefix="[☛]")]
        if not result.result[1].startswith('magnet'):
            # Allow jackett redirect to magnet uri.
            resp = await BaseSearch.session.get(result.result[1],
                                                allow_redirects=False)
            result.result[1] = resp.headers['Location']

        if self.option('open'):
            return result.open()
        if stream:
            await stream_torrent(result.result[1])
        self.line(result.result[1])


class BotCommand(SearcherCommand):
    """Launch bot

    bot

        {--token=? : Bot token. Ask BotFather}
    """
    async def run_bot(self):
        await self.setup_sessions()
        if 'bot' not in BaseSearch.config.sections():
            BaseSearch.config.add_section('bot')

        token = os.getenv(
            'KATCR_TOKEN',
            BaseSearch.config['bot'].get('token', self.option('token')))

        await BotHandler(self).start(token)
        await self.teardown_sessions()

    def handle(self):
        return asyncio.run(self.run_bot())


class CLICommand(SearcherCommand):
    """Search in multiple torrent sites.

    search

        {search : Search term}

        {--pages=1 : Pages to search on search engines}
        {--engine=?* : Engines}
        {--nointeractive=? : Print resuls directly to stdout}
        {--open=? : Open selected magnet with xdg-open}
        {--stream=? : Stream with torrentstream, uses PLAYER env or xdg-open }

    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.logger = None

    def handle(self):
        """Handler."""
        if self.option('engine'):
            engine_names = self.option('engine')
        else:
            engine_names = [
                "Katcr", "ThePirateBay", "Eztv", "NyaaSi", "Skytorrents"
            ]
        is_interactive = not self.option('nointeractive')

        self.line(f'<info>Starting search on {", ".join(engine_names)}</info>')

        coro = self.search(engine_names, self.argument('search'),
                           self.option('pages'), is_interactive,
                           self.option('stream'))
        return asyncio.run(coro)


def main():
    """Main entry point"""
    application = Application()
    application.add(CLICommand())
    application.add(BotCommand())
    application.run()



import html.parser
from .base import BaseSearch


class Skytorrents(BaseSearch):
    """Skytorrents.in torrent search engine."""

    proxy_name = ''
    url = 'https://skytorrents.lol/'
    url_format = '{}{}/?query={}&page={}'

    @staticmethod
    def parse_magnet(magnet):
        """Skytorrents magnets are html-scaped"""
        return BaseSearch.parse_magnet(html.parser.unescape(magnet))



import os
from feedparser import parse
from .base import BaseSearch

HOST = os.getenv('JACKETT_HOST')
APIKEY = os.getenv('JACKETT_APIKEY')


class Jackett(BaseSearch):
    """Use jackett's all search engines feature"""

    proxy_name = None
    url_format = '{}{}&q={}&p={}'

    @property
    def url(self):
        """Return formatted url extracting host and key from config or env."""
        if not self.option:
            self.option = 'all'
        if not self.config.has_section('jackett'):
            self.config.add_section('jackett')
        host = os.getenv('JACKETT_HOST',
                         self.config.get('jackett', 'host', fallback=None))
        apikey = os.getenv('JACKETT_APIKEY',
                           self.config.get('jackett', 'apikey', fallback=None))
        return (f"{host}/api/v2.0/indexers/"
                f"{self.option}/results/torznab?apikey={apikey}")

    async def search_site(self, url):
        """Search jackett using feedparser"""
        return [[item.title, item.link] for item in parse(url).entries]



from contextlib import suppress
import urllib.parse
import re
import abc
import torrentmirror
import aiohttp

CLDS = 'cloudflare', 'cloudfront'
MAG = re.compile(
    r"(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+")


class CloudFlareError(Exception):
    """Cloud flare detected"""


class BaseSearch(metaclass=abc.ABCMeta):
    """Base Search."""
    session = None
    logger = None

    def __init__(self, option=None):
        self.option = option

    @abc.abstractproperty
    def url(self):
        """Search on given url by default."""

    @abc.abstractproperty
    def proxy_name(self):
        """Proxy name to use from torrentmirror."""

    @abc.abstractproperty
    def url_format(self):
        """Url formatter to add to base_url."""

    async def search_magnets(self, query: str, page: int):
        """Return the list of magnets from a specific page.

        warning: paging may produce strange results if the first available
        proxy changes between searches.

        Arguments

            query: Query string
            page: Page to search on
        """
        proxies = []
        try:
            all_proxies = torrentmirror.get_proxies()
        except Exception:
            all_proxies = {}
            self.logger.debug('cant_get_proxies')

        proxies = [
            a['link'] for a in all_proxies.get(self.proxy_name, {})
            if a.get('status') == 'ONLINE'
        ]
        self.logger.debug("Got proxies: %s", proxies)
        proxies.insert(0, self.url)

        for site in proxies:
            # First site to get results wins.
            http = '' if site.startswith('http') else 'http://'
            url = self.url_format.format(http, site, query, page)
            self.logger.debug("Searching in %s (%s)", site, url)
            if res := await self.search_site(url):
                return res

        return []

    async def search_site(self, url):
        with suppress(aiohttp.ClientError, CloudFlareError):
            response = await self.session.get(url)
            text = await response.text()

            if any(reserved in text for reserved in CLDS):
                raise CloudFlareError()

            return [a async for a in self.get_torrents(response)]

    @classmethod
    async def get_torrents(cls, data):
        """Get torrents from a site. data will be a response."""
        data = await data.text()
        results = (a for a in MAG.findall(data) if a.startswith('magnet'))
        for result in results:
            result = cls.parse_magnet(result)
            if result:
                yield result

    @staticmethod
    def parse_magnet(magnet):
        """Parse magnet, only returns identifiable magnets."""
        result = urllib.parse.parse_qs(magnet)
        if result and result.get('dn'):
            return next(iter(result.get('dn', [])), ''), magnet

    async def search(self, query: str, pagelen: int = 1):
        """Manage complete search for specified pages."""
        results = []
        for page in range(0, pagelen):
            self.logger.debug("Getting page %s", page)
            results.extend(await self.search_magnets(query, page + 1))
        return results



from .base import BaseSearch
from urllib.parse import quote

TRACKERS = ("udp://tracker.coppersurfer.tk:6969/announce",
            "udp://9.rarbg.me:2850/announce", "udp://9.rarbg.to:2920/announce",
            "udp://tracker.opentrackr.org:1337",
            "udp://tracker.leechers-paradise.org:6969/announce")

TRACKERS = list(quote(t, safe='') for t in TRACKERS)


class ThePirateBay(BaseSearch):
    """ThePirateBay search engine.

    Search on any of the pirate bay proxies or thepiratebay itself if
    available. Extract torrents, description and return it tabulated and ready
    to be printed
    """

    proxy_name = 'The Pirate Bay'
    url_format = '{}{}q.php?q={}&cat='
    url = 'https://apibay.org/'

    async def get_torrents(self, response):
        result = await response.json()
        for mag in result:
            yield [
                mag['name'],
                (f"magnet:?xt=urn:btih:{mag['info_hash']}"
                 f"&dn={quote(mag['name'])}&tr={'&tr='.join(TRACKERS)}")
            ]



from aiogram import Bot, Dispatcher, types


class BotHandler:
    """Main bothandler class"""
    def __init__(self, handler):
        """Set parent handler to delegate search function."""
        self.handler = handler

    async def start_handler(self, event: types.Message):
        """Start command"""
        await event.answer(
            (f"Hello, {event.from_user.get_mention(as_html=True)} 👋!"
             f" I'm katbot, use me to search on many torrent search engines."
             f" Use the /search <your_search> command with your query and I'll"
             f" return some results"),
            parse_mode=types.ParseMode.HTML,
        )

    async def search_handler(self, event: types.Message):
        """Search in providers."""
        provs = ['ThePirateBay', 'Eztv', 'NyaaSi', 'Skytorrents']
        response = await self.handler.search(provs, event.text[8:], 1, 0, 0)
        for res in response:
            await event.answer(f'{res.result[0]} &gt; {res.result[1]}',
                               parse_mode=types.ParseMode.HTML)

    async def start(self, token):
        """Start bot."""
        bot = Bot(token=token)
        try:
            disp = Dispatcher(bot=bot)
            register = disp.register_message_handler
            register(self.start_handler, commands={"start"})
            register(self.search_handler, commands={"search"})
            await disp.start_polling()
        finally:
            await bot.close()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['katcr', 'katcr.engines']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.2,<4.0.0',
 'cleo',
 'cutie>=0.2.2,<0.3.0',
 'feedparser>=6.0.2,<7.0.0',
 'pytest-asyncio>=0.14.0,<0.15.0',
 'torrentmirror',
 'xdg>=5.0.1,<6.0.0']

extras_require = \
{':extra == "bot"': ['aiogram>=2.11.2,<3.0.0']}

entry_points = \
{'console_scripts': ['bot = katcr:bot', 'katcr = katcr:main']}

setup_kwargs = {
    'name': 'katcr',
    'version': '3.0.7',
    'description': 'KickassTorrents CLI and Telegram bot',
    'long_description': None,
    'author': 'David Francos',
    'author_email': 'opensource@davidfrancos.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
